console.log('Loading mynote_lambda function');

var AWS = require("aws-sdk");

var m_dynamodbDoc = null;
var table = "Notes";
var regin = "<リージョン名>";


exports.handler = function(event, context) {
	switch (event.func) {
	case 'getNoteList':
		execGetNoteList(event, context);
		break;
	case 'updateNote':
		execUpdateNote(event, context);
		break;
	case 'addNote':
		execUpdateNote(event, context);
		break;
	case 'clearAll':
		execClearAll(event, context);
		break;
	case 'deleteNote':
		execDeleteNote(event, context);
		break;
	default:
		console.log('unknown event.func=' + event.func);
		break;
	}
};

function execGetNoteList(event, context) {
	openDb();
	getNoteList(context.identity.cognitoIdentityId, function(isSuccess, data) {
		if (isSuccess) {
			context.succeed({"operation" : "new", "notes" : data});
		} else {
			context.fail("failed getNoteList: " + data);
		}
	});
}

function execUpdateNote(event, context) {
	openDb();
	updateNote(context.identity.cognitoIdentityId, event.param, function(isSuccess, data) {
		if (isSuccess) {
			context.succeed({"operation" : "add", "note" : data});
		} else {
			context.fail("failed updateNote: " + data);
		}
	});
}

function execClearAll(event, context) {
	openDb();
	clearAll(context.identity.cognitoIdentityId, function(isSuccess, data) {
		if (isSuccess) {
			context.succeed({"operation" : "new", "notes" : []});
		} else {
			context.fail("failed clearAll: " + data);
		}
	});
}

function execDeleteNote(event, context) {
	openDb();
	deleteNote(context.identity.cognitoIdentityId, event.param.date_time, function(isSuccess, data) {
		if (isSuccess) {
			context.succeed({"operation" : "delete", "note" : data});
		} else {
			context.fail("failed deleteNote: " + data);
		}
	});
}

function openDb() {
	if (m_dynamodbDoc === null) {
		console.log('start open dynamoDB');
		AWS.config.update({
		  region: region,
		  endpoint: "https://dynamodb." + region + ".amazonaws.com"
		});
		
		m_dynamodbDoc = new AWS.DynamoDB.DocumentClient();
		console.log('end open dynamoDB');
	}
}

function getNoteList(userId, callback) {
	console.log("userId", userId);
	var params = {
		"TableName" : table,
		"KeyConditionExpression": "user_id = :userId",
		"ExpressionAttributeValues": {
			":userId": userId
		}
	};
	console.log("query param ", JSON.stringify(params));
	m_dynamodbDoc.query(params, function(err, data) {
		var notes = [];
		if (err) {
			var errStr = "Unable to query. Error:" + JSON.stringify(err, null, 2);
			console.log(errStr);
			callback(false, errStr);
		} else {
			console.log("Query succeeded.");
			var itemSize = data.Items.length;
			if (itemSize === 0) {
				callback(true, notes);
			} else {
				data.Items.forEach(function(item) {
					console.log(" -", JSON.stringify(item));
					var note = {
						"user_id" : item.user_id,
						"date_time" : item.date_time,
						"text_type" : item.text_type,
						"text" : item.text
					};
					notes.push(note);
				});
				callback(true, notes);
			}
		}
	});
}

function updateNote(userId, item, callback) {
	var params = {
		"TableName" : table,
		"Item" : {
			"user_id" : userId,
			"date_time" : item.date_time,
			"text_type" : item.text_type,
			"text" : item.text
		}
	};
	
	console.log("Adding a new item...", JSON.stringify(params));
	m_dynamodbDoc.put(params, function(err, data) {
	    if (err) {
	        console.error("Unable to add item. Error JSON:", JSON.stringify(err, null, 2));
    	    callback(false, JSON.stringify(err));
	    } else {
	        console.log("Added item:", JSON.stringify(data, null, 2));
    	    callback(true, params.Item);
	    }
	});
}

function deleteNote(userId, dateTime, callback) {
	var data = {
		user_id: userId,
		date_time: dateTime
	};
	var dataList= [];
	dataList.push(data);
	deleteItems(dataList, function(isSuccess, deletedItems) {
	    if (isSuccess) {
	        callback(isSuccess, deletedItems[0]);
	    } else {
	        callback(isSuccess, deletedItems);
	    }
	});
}

function clearAll(userId, callback) {
	var params = {
		"TableName" : table,
		"KeyConditionExpression": "user_id = :userId",
		"ExpressionAttributeValues": {
			":userId": userId
		}
	};
	console.log("query param ", JSON.stringify(params));
	m_dynamodbDoc.query(params, function(err, data) {
		var notes = [];
		if (err) {
			var errStr = "Unable to query. Error: " + JSON.stringify(err, null, 2);
			console.log(errStr);
			callback(false, errStr);
		} else {
			console.log("Query succeeded.");
			deleteItems(data.Items, callback);
		}
	});
}

function deleteItems(items, callback) {
	params = {
		"RequestItems": {
		}
	};
	var deletedItems = [];
	params.RequestItems[table] = [];
	console.log("test ", JSON.stringify(params));
	items.forEach(function(item) {
		console.log(" -", JSON.stringify(item));
		var request = {
			"DeleteRequest" : {
				"Key": {
					"user_id": item.user_id,
					"date_time" : item.date_time
				}
			}
		};
		params.RequestItems[table].push(request);
		deletedItems.push({
			"user_id": item.user_id,
			"date_time" : item.date_time
		});
	});
	console.log("delete req:", JSON.stringify(params));
	m_dynamodbDoc.batchWrite(params, function(err, data) {
		if (err) {
			var errStr = "Unable to batchWriteItem. Error: " + JSON.stringify(err, null, 2);
			console.log(errStr);
			callback(false, errStr);
		} else {
			console.log("batchWriteItem succeeded.");
			callback(true, deletedItems);
		}
	});
}